package heuristicmonitor;

import java.lang.reflect.*;
import java.io.*;
import java.text.*;
import java.util.*;
import com.sonicsw.mf.common.metrics.*;
import com.sonicsw.mf.common.runtime.*;
import com.sonicsw.mf.jmx.client.*;
import com.sonicsw.mx.util.*;

import javax.management.*;
/*
 * Sample
 * @author rares
 */
public class MgmtCLI extends CLI {

    private static Hashtable m_commands = new Hashtable();
    private static String m_context = "";
    private static JMSConnectorClient m_connector;    // notification support
    private static NotificationListener m_notificationListener = new NotificationReporter();
    private static HashMap m_notificationFilters = new HashMap();    // some useful constants
    private static final Object[] EMPTY_OBJECT_ARRAY = new Object[0];
    private static final String[] EMPTY_STRING_ARRAY = new String[0];
    private static final Class[] STRING_PARAMTYPES = new Class[]{String.class};
    private static SimpleDateFormat SIMPLE_DATE_FORMAT;
    

    static {
        // override the parent class object
        m_cliClassObject = MgmtCLI.class;
        try {
            addCommand("exit",
                    m_cliClassObject.getDeclaredMethod("exit", new Class[0]),
                    m_cliClassObject.getDeclaredMethod("exitHelp", new Class[0]));
            addCommand("cc",
                    m_cliClassObject.getDeclaredMethod("cc", new Class[]{String.class}),
                    m_cliClassObject.getDeclaredMethod("ccHelp", new Class[0]));
            addCommand("show attributes",
                    m_cliClassObject.getDeclaredMethod("showAttributes", new Class[]{String.class}),
                    m_cliClassObject.getDeclaredMethod("showAttributesHelp", new Class[0]));
            addCommand("show operations",
                    m_cliClassObject.getDeclaredMethod("showOperations", new Class[]{String.class}),
                    m_cliClassObject.getDeclaredMethod("showOperationsHelp", new Class[0]));
            addCommand("show notifications",
                    m_cliClassObject.getDeclaredMethod("showNotifications", new Class[]{String.class}),
                    m_cliClassObject.getDeclaredMethod("showNotificationsHelp", new Class[0]));
            addCommand("show metrics",
                    m_cliClassObject.getDeclaredMethod("showMetrics", new Class[]{String.class}),
                    m_cliClassObject.getDeclaredMethod("showMetricsHelp", new Class[0]));
            addCommand("show status",
                    m_cliClassObject.getDeclaredMethod("showStatus", new Class[]{String.class}),
                    m_cliClassObject.getDeclaredMethod("showStatusHelp", new Class[0]));
            addCommand("show enabled",
                    m_cliClassObject.getDeclaredMethod("showEnabled", new Class[]{String.class}),
                    m_cliClassObject.getDeclaredMethod("showEnabledHelp", new Class[0]));
            addCommand("show log",
                    m_cliClassObject.getDeclaredMethod("showLog", new Class[]{String.class}),
                    m_cliClassObject.getDeclaredMethod("showLogHelp", new Class[0]));
            addCommand("get",
                    m_cliClassObject.getDeclaredMethod("get", new Class[]{String.class, String.class}),
                    m_cliClassObject.getDeclaredMethod("getHelp", new Class[0]));
            addCommand("set",
                    m_cliClassObject.getDeclaredMethod("set", new Class[]{String.class, String.class, String.class}),
                    m_cliClassObject.getDeclaredMethod("setHelp", new Class[0]));
            addCommand("enable",
                    m_cliClassObject.getDeclaredMethod("enable", new Class[]{String.class, String.class}),
                    m_cliClassObject.getDeclaredMethod("enableHelp", new Class[0]));
            addCommand("disable",
                    m_cliClassObject.getDeclaredMethod("disable", new Class[]{String.class, String.class}),
                    m_cliClassObject.getDeclaredMethod("disableHelp", new Class[0]));
            addCommand("capture",
                    m_cliClassObject.getDeclaredMethod("capture", new Class[]{String.class, String.class}),
                    m_cliClassObject.getDeclaredMethod("captureHelp", new Class[0]));
            addCommand("invoke",
                    m_cliClassObject.getDeclaredMethod("invoke", new Class[]{String.class, String.class, String.class}),
                    m_cliClassObject.getDeclaredMethod("invokeHelp", new Class[0]));
            addCommand("subscribe",
                    m_cliClassObject.getDeclaredMethod("subscribe", new Class[]{String.class, String.class}),
                    m_cliClassObject.getDeclaredMethod("subscribeHelp", new Class[0]));
            addCommand("unsubscribe",
                    m_cliClassObject.getDeclaredMethod("unsubscribe", new Class[]{String.class, String.class}),
                    m_cliClassObject.getDeclaredMethod("unsubscribeHelp", new Class[0]));
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } // should not happen !

        // date format setup
        SIMPLE_DATE_FORMAT = new SimpleDateFormat();
        SIMPLE_DATE_FORMAT.applyPattern("yy/MM/dd kk:mm:ss");
    }

    /*
     * JMX management application
     * Copyright 2001-2008, Progress Software Corporation -  All Rights Reserved
     * Management Sample code
     *
     * The MgmtCLI is a sample JMX management application that demonstrates a wide
     * range of generic management capabilities exposed through the Sonic Management
     * Framework's openAPI. The application provides a simple command line interface
     * that can be easily extended to add new more specific commands.
     *
     * The following commands are supported by this application:
     *
     *  > show {attributes|operations|notifications|metrics} [<domain>.<container>:<id>]
     *  > show subscriptions [<domain>.<container>:<id>]
     *  > show enabled [<domain>.<container>:<id>]
     *  > show status [<see below>]
     *     - <domain>
     *     - <domain>.<container>
     *     - <domain>.<container>:<id>
     *
     *  > get [<domain>.<container>:<id>] <attribute name>{,<attribute name>...}
     *  > set [<domain>.<container>:<id>] <attribute name> <value>
     *
     *  > invoke [<domain>.<container>:<id>] <operation name> {param1,{param2...}}
     *
     *  > subscribe [<domain>.<container>:<id>] <notification type>{,<notification type>...}
     *  > unsubscribe [<domain>.<container>:<id>] <notification type>{,<notification type>...}
     *
     *  > enable [<domain>.<container>:<id>] <metric name>{,<metric name>...}
     *  > disable [<domain>.<container>:<id>] <metric name>{,<metric name>...}
     *  > capture [<domain>.<container>:<id>] <metric name>{,<metric name>...}
     *
     *  > cc [<see below>] (change context)
     *     - <domain>
     *     - <domain>.<container>
     *     - <domain>.<container>:<id>
     *
     *  > exit
     */
    public MgmtCLI(String ConnectionURLs, String DefaultUser, String DefaultPassword) {

        // create a hashtable with any entries applicable that are allowed by a SonicMQ
        // topic connection factory
        try {
            Hashtable env = new Hashtable();
            env.put("ConnectionURLs", ConnectionURLs);
            env.put("DefaultUser", DefaultUser);
            env.put("DefaultPassword", DefaultPassword);

            // create a JMX/JMS connection to the SonicMQ broker used for management communications
            JMSConnectorAddress address = new JMSConnectorAddress(env);
            m_connector = new JMSConnectorClient();
            m_connector.connect(address);
            m_connector.setConnectionListener(new IConnectionListener() {

                @Override
                public void onReconnect(String localRoutingNode) {
                    System.err.println("\nReconnected");
                }

                @Override
                public void onDisconnect() {
                    System.err.println("\nDisconnected");
                }

                @Override
                public void onFailure(Exception e) {
                    System.err.println("\nCommunication failure, trace follows...\n");
                    System.err.println(exceptionToString(e));
                    System.err.println("\nExiting...\n");
                    System.exit(1);
                }

                @Override
                public void onNotificationListenerRenewalFailure(Exception e) {
                    System.err.println("\nNotification listener renewal failure, trace follows...\n");
                    System.err.println(exceptionToString(e));
                }
            });

        } catch (Exception e) {
            e.printStackTrace();
        }


    } //constructor

    public void main(String[] args) {
        // create a hashtable with any entries applicable that are allowed by a SonicMQ
        // topic connection factory
        try {
            if (args.length < 1) {
                printUsage();
            }
            Hashtable env = new Hashtable();
            env.put("ConnectionURLs", args[0]);
            env.put("DefaultUser", args.length > 1 ? args[1] : "");
            env.put("DefaultPassword", args.length > 2 ? args[2] : "");

            final String[] cmdArgs = args;

            // create a JMX/JMS connection to the SonicMQ broker used for management communications
            JMSConnectorAddress address = new JMSConnectorAddress(env);
            m_connector = new JMSConnectorClient();
            m_connector.connect(address);
            m_connector.setConnectionListener(new IConnectionListener() {

                @Override
                public void onReconnect(String localRoutingNode) {
                    System.err.println("\nReconnected to " + cmdArgs[0]);
                    redoPrompt();
                }

                @Override
                public void onDisconnect() {
                    System.err.println("\nDisconnected from " + cmdArgs[0]);
                    redoPrompt();
                }

                @Override
                public void onFailure(Exception e) {
                    System.err.println("\nCommunication failure, trace follows...\n");
                    System.err.println(exceptionToString(e));
                    System.err.println("\nExiting...\n");
                    System.exit(1);
                }

                @Override
                public void onNotificationListenerRenewalFailure(Exception e) {
                    System.err.println("\nNotification listener renewal failure on " + cmdArgs[0] + ", trace follows...\n");
                    System.err.println(exceptionToString(e));
                    System.err.println();
                    redoPrompt();
                }
            });

            // best practice is to add a shutdown hook that will attempt to clean up the open connection
            Runtime.getRuntime().addShutdownHook(new Thread() {

                @Override
                public void run() {
                    //MgmtCLI.exit();
                }
            });

            new Thread("Command Line Parser") {

                @Override
                public void run() {
                    readParseExecute();
                }
            }.start();
            Thread.currentThread().join();
        } catch (Exception e) {
            e.printStackTrace();
        }
    } //main

    private static void printUsage() {
        System.out.println();
        System.out.println("Usage: MgmtCLI <url> [user [password]]");
        System.out.println();
        System.out.println("Where: <url>      SonicMQ connectiuon url to the message broker used for");
        System.out.println("                  management communications");
        System.out.println("       <user>     If communications security is enabled, the SonicMQ");
        System.out.println("                  user name");
        System.out.println("       <password> If communications security is enabled, the SonicMQ");
        System.out.println("                  password");

        System.exit(1);
    }

    //
    // command implementations
    //
    public String exit() {
        m_connector.disconnect();
        Runtime.getRuntime().halt(0);
        return null;
    }

    public String cc(String context) {
        // if a non empty context is provided, validate the context
        if (context.length() > 0) {
            try {
                ObjectName name = null;
                if (context.indexOf(":ID=") > -1) // if its a full component context, check the status (assume it must be online or whats the point
                // in setting the context to it)
                {
                    name = new ObjectName(context);
                } else if (context.indexOf('.') > -1) // if its a container context, check the status of the AGENT in that container (again, assume it
                // must be online or whats the point in setting the context to it)
                {
                    name = new ObjectName(context + ":ID=AGENT");
                } else // it must be a domain and hence check the status of the Agent manager  (again, assume it
                // must be online or whats the point in setting the context to it)
                {
                    name = new ObjectName(context + ".AGENT MANAGER:ID=AGENT MANAGER");
                }
                Short status = (Short) m_connector.getAttribute(name, "State");
            } catch (Exception e) {
                return "error: Failed to verify context status, trace follows...\n" + exceptionToString(e);
            }
        }

        m_context = context;
        setPrompt(m_context + DEFAULT_PROMPT);
        return null;
    }

    public static String showAttributes(String context) {
        if (context == null) {
            context = m_context;
        }
        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";        // get all the attributes
        }
        try {
            ObjectName name = new ObjectName(context);
            MBeanAttributeInfo[] attributeInfos = m_connector.getMBeanInfo(name).getAttributes();

            StringBuffer results = new StringBuffer();
            for (int i = 0; i < attributeInfos.length; i++) {
                if (i > 0) {
                    results.append('\n');
                }
                results.append(attributeInfos[i].getName()).append("\n- ").append(attributeInfos[i].getDescription());
            }

            return results.toString();
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public static String showOperations(String context) {
        if (context == null) {
            context = m_context;
        }
        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";        // get all the operations
        }
        try {
            ObjectName name = new ObjectName(context);
            MBeanOperationInfo[] operationInfos = m_connector.getMBeanInfo(name).getOperations();

            StringBuffer results = new StringBuffer();
            for (int i = 0; i < operationInfos.length; i++) {
                if (i > 0) {
                    results.append('\n');
                }
                results.append(operationInfos[i].getName()).append('(');
                MBeanParameterInfo[] params = operationInfos[i].getSignature();
                for (int j = 0; j < params.length; j++) {
                    if (j > 0) {
                        results.append(", ");
                    // format the type
                    }
                    String paramType = params[j].getType();
                    if (paramType.startsWith("[L")) {
                        paramType = paramType.substring(2, paramType.length() - 1) + "[]";
                    }
                    results.append(paramType);
                }
                results.append(')').append("\n- ").append(operationInfos[i].getDescription());
            }

            return results.toString();
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public static String showNotifications(String context) {
        if (context == null) {
            context = m_context;
        }
        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";        // get all the notifications
        }
        try {
            ObjectName name = new ObjectName(context);
            MBeanNotificationInfo[] notificationInfos = m_connector.getMBeanInfo(name).getNotifications();

            StringBuffer results = new StringBuffer();
            for (int i = 0; i < notificationInfos.length; i++) {
                if (notificationInfos[i] != null) {
                    String[] notificationNames = notificationInfos[i].getNotifTypes();
                    if (notificationNames != null) {
                        if (i > 0) {
                            results.append('\n');
                        }
                        for (int j = 0; j < notificationNames.length; j++) {
                            if (j > 0) {
                                results.append('.');
                            }
                            results.append(notificationNames[j]);
                        }
                        results.append("\n- ").append(notificationInfos[i].getDescription());
                    }
                }
            }

            return results.toString();
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public static String showMetrics(String context) {
        if (context == null) {
            context = m_context;
        }
        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";        // get all the metrics
        }
        try {
            ObjectName name = new ObjectName(context);
            IMetricInfo[] metricInfos = (IMetricInfo[]) m_connector.invoke(name, "getMetricsInfo", EMPTY_OBJECT_ARRAY, EMPTY_STRING_ARRAY);

            StringBuffer results = new StringBuffer();
            for (int i = 0; i < metricInfos.length; i++) {
                if (i > 0) {
                    results.append('\n');
                }
                results.append(metricInfos[i].getMetricIdentity());
                results.append(" (").append("type=").append(IValueType.DESCRIPTIONS[metricInfos[i].getValueType()]);
                results.append(", ").append("instance=").append(metricInfos[i].isInstanceMetric());
                results.append(", ").append("dynamic=").append(metricInfos[i].isDynamic());
                results.append(")\n- ").append(metricInfos[i].getDescription());
            }

            return results.toString();
        } catch (ReflectionException re) {
            Exception target = re.getTargetException();
            return (target instanceof NoSuchMethodException ? "*no metrics*" : exceptionToString(re));
        } catch (MBeanException me) {
            Exception target = me.getTargetException();
            return exceptionToString(me.getTargetException());
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public String showStatus(String context) {
        if (context == null) {
            context = m_context;
        }
        if (context.length() == 0) {
            return "error: Invalid working context for command.";        // get the status
        }
        try {
            if (context.indexOf(":ID=") > -1) // if its a full component context, check the status (assume it must be online or whats the point
            // in setting the context to it)
            {
                return showComponentStatus(context);
            } else if (context.indexOf('.') > -1) // if its a container context, check the status of the AGENT in that container (again, assume it
            // must be online or whats the point in setting the context to it)
            {
                return showContainerStatus(context);
            } else // it must be a domain and hence check the status of the Agent manager  (again, assume it
            // must be online or whats the point in setting the context to it)
            {
                return showDomainStatus(context);
            }
        } catch (Exception e) {
            return "error: " + exceptionToString(e);
        }
    }

    public String showLog(String context) {
        if (context == null) {
            context = m_context;
        }
        if (context.length() == 0 || isComponentContext(context) || context.indexOf('.') == -1) {
            return "error: Invalid working context for command.";        // get the tail of the log file
        }
        try {
            ObjectName name = new ObjectName(context + ":ID=AGENT");
            Object[] params = {null, null};
            String[] signatures = {Long.class.getName(), Long.class.getName()};
            return (String) m_connector.invoke(name, "getLogExtract", params, signatures);
        } catch (Exception e) {
            return "error:" + exceptionToString(e);
        }
    }

    public String showEnabled(String context) {
        if (context == null) {
            context = m_context;
        }
        if (context.length() == 0 /*|| !isComponentContext(context)*/ ) {
            return "error: Invalid working context for command.";        // get the enabled metrics
        }
        try {
            ObjectName name = new ObjectName(context);
            Object[] params = {null};
            String[] signatures = {"[Lcom.sonicsw.mf.common.metrics.IMetricIdentity;"};
            IMetricIdentity[] metricIds = (IMetricIdentity[]) m_connector.invoke(name, "getEnabledMetrics", params, signatures);

            StringBuffer results = new StringBuffer();
            if (metricIds.length > 0) {
                for (int i = 0; i < metricIds.length; i++) {
                    if (i > 0) {
                        results.append('\n');
                    }
                    results.append(metricIds[i].getName());
                }
            } else {
                results.append("error: *no enabled metrics*");
            }
            return results.toString();
        } catch (ReflectionException re) {
            Exception target = re.getTargetException();
            return (target instanceof NoSuchMethodException ? "error: *no enabled metrics*" : "error: " + exceptionToString(re));
        } catch (MBeanException me) {
            Exception target = me.getTargetException();
            return "error: " + exceptionToString(me.getTargetException());
        } catch (Exception e) {
            return "error: " + exceptionToString(e);
        }
    }

    public static String get(String param1, String param2) {
        if (param1 == null) {
            return getHelp();        // if no second arg, assume no context is provided
        }
        String context = param2 == null ? m_context : param1;

        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";
        }
        String names = param2 == null ? param1 : param2;

        StringTokenizer st = new StringTokenizer(names, ", ");
        String[] attributeNames = new String[st.countTokens()];
        for (int i = 0; i < attributeNames.length; i++) {
            attributeNames[i] = st.nextToken();        // get all the attribute names
        }
        try {
            ObjectName name = new ObjectName(context);
            AttributeList attributes = m_connector.getAttributes(name, attributeNames);

            StringBuffer results = new StringBuffer();
            for (int i = 0; i < attributes.size(); i++) {
                Attribute attribute = (Attribute) attributes.get(i);
                if (i > 0) {
                    results.append('\n');
                }
                results.append(attribute.getName()).append('=').append(attribute.getValue());
            }

            return results.toString();
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public static String set(String param1, String param2, String param3) {
        if (param1 == null && param2 == null) {
            return setHelp();        // if no third arg, assume no context is provided
        }
        String context = param3 == null ? m_context : param1;

        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";
        }
        String attributeName = param3 == null ? param1 : param2;
        String attributeValue = param3 == null ? param2 : param3;

        // set the attribute
        try {
            ObjectName name = new ObjectName(context);

            // get the attribute first, then the datatype can be determined
            // and a new value of the same type constructed
            Object oldValue = m_connector.getAttribute(name, attributeName);
            Object newValue = convertDatatype(oldValue.getClass(), attributeValue);

            // now set the attribute
            m_connector.setAttribute(name, new Attribute(attributeName, newValue));

            return null;
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public static String enable(String param1, String param2) {
        if (param1 == null) {
            return enableHelp();        // if no second arg, assume no context is provided
        }
        String context = param2 == null ? m_context : param1;

        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";
        }
        String metrics = param2 == null ? param1 : param2;
        StringTokenizer st = new StringTokenizer(metrics, ",");
        IMetricIdentity[] metricIds = new IMetricIdentity[st.countTokens()];
        for (int i = 0; i < metricIds.length; i++) {
            metricIds[i] = MetricsFactory.createMetricIdentity(st.nextToken());        // enable the metrics
        }
        try {
            ObjectName name = new ObjectName(context);
            m_connector.invoke(name, "enableMetrics", new Object[]{metricIds}, new String[]{IMetricIdentity[].class.getName()});

            return null;
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public static String disable(String param1, String param2) {
        if (param1 == null) {
            return disableHelp();        // if no second arg, assume no context is provided
        }
        String context = param2 == null ? m_context : param1;

        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";
        }
        String metrics = param2 == null ? param1 : param2;
        StringTokenizer st = new StringTokenizer(metrics, ",");
        IMetricIdentity[] metricIds = new IMetricIdentity[st.countTokens()];
        for (int i = 0; i < metricIds.length; i++) {
            metricIds[i] = MetricsFactory.createMetricIdentity(st.nextToken());        // disable the metrics
        }
        try {
            ObjectName name = new ObjectName(context);
            m_connector.invoke(name, "disableMetrics", new Object[]{metricIds}, new String[]{IMetricIdentity[].class.getName()});

            return null;
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public String capture(String param1, String param2) {
        if (param1 == null) {
            return "error: " + captureHelp();        // if no second arg, assume no context is provided
        }
        String context = param2 == null ? m_context : param1;

        if (context.length() == 0 || !isComponentContext(context)) {
            return "error: " + "Invalid working context for command.";
        }
        String metrics = param2 == null ? param1 : param2;

        StringTokenizer st = new StringTokenizer(metrics, ",");
        IMetricIdentity[] metricIds = new IMetricIdentity[st.countTokens()];
        for (int i = 0; i < metricIds.length; i++) {
            metricIds[i] = MetricsFactory.createMetricIdentity(st.nextToken());        // get the requested metrics
        }
        try {
            ObjectName name = new ObjectName(context);
            IMetricsData metricsData = (IMetricsData) m_connector.invoke(name, "getMetricsData", new Object[]{metricIds, Boolean.FALSE}, new String[]{IMetricIdentity[].class.getName(), Boolean.class.getName()});
            IMetric[] metricValues = metricsData.getMetrics();

            if (metricValues.length == 0) {
                return "error: " + "*no metrics*";
            }
            StringBuffer results = new StringBuffer();
            //results.append('(').append("timestamp=").append(new Date(metricsData.getCurrencyTimestamp())).append(')');

            for (int i = 0; i < metricValues.length; i++) //results.append("\n- ").append(metricValues[i].getMetricIdentity()).append('=').append(metricValues[i].getValue());
            {
                if (!metricValues[i].getMetricIdentity().getName().isEmpty()) {
                    results.append(metricValues[i].getMetricIdentity()).append('=').append(metricValues[i].getValue()).append('\n');
                }
            }
            return results.toString();
        } catch (Exception e) {
            return "error: " + exceptionToString(e);
        }
    }

    public static String invoke(String param1, String param2, String param3) {
        if (param1 == null) {
            return invokeHelp();        // if no second arg, assume no context is provided
        }
        String context = isComponentContext(param1) ? param1 : m_context;

        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";
        }
        String operation = isComponentContext(param1) ? param2 : param1;
        String paramList = isComponentContext(param1) ? param3 : param2;

        try {
            ObjectName name = new ObjectName(context);

            // have the correct number of parameters been provided for the operation
            // (need to get the meta data for the operation)
            MBeanOperationInfo[] operationInfos = m_connector.getMBeanInfo(name).getOperations();
            MBeanOperationInfo operationInfo = null;
            for (int i = 0; i < operationInfos.length; i++) {
                if (operationInfos[i].getName().equals(operation)) {
                    operationInfo = operationInfos[i];
                    break;
                }
            }

            // was the operation found ?
            if (operationInfo == null) {
                return "*no such operation*";            // now convert the parameters to the signature types
            }
            MBeanParameterInfo[] paramTypes = operationInfo.getSignature();
            Object[] params = new Object[paramTypes.length];
            String[] signature = new String[paramTypes.length];
            StringTokenizer st = new StringTokenizer(paramList == null ? "" : paramList, ",", true);
            boolean gotDelimiter = false;
            for (int i = 0; i < paramTypes.length;) {
                if (st.hasMoreTokens()) {
                    String token = st.nextToken();
                    if (token.equals(",")) {
                        if (gotDelimiter) // then we have an empty param so we use null
                        {
                            token = null;
                        } else {
                            gotDelimiter = true;
                            continue;
                        }
                    }
                    gotDelimiter = false;
                    params[i] = token;
                    signature[i] = paramTypes[i].getType();
                    if (params[i] != null && !paramTypes[i].getType().equals(params[i].getClass().getName())) {
                        params[i] = convertDatatype(m_cliClassObject.getClassLoader().loadClass(signature[i]), (String) params[i]);
                    }
                    i++;
                }
            }

            // invoke the operation
            Object returnValue = m_connector.invoke(name, operation, params, signature);

            // if no return value simply return
            if (operationInfo.getReturnType().equals(Void.class.getName())) {
                return null;
            }
            StringBuffer results = new StringBuffer("result=");

            if (returnValue == null) // if return value null, reflect that fact
            {
                results.append("null");
            } else if (returnValue instanceof Object[]) // its an array
            {
                Object[] returnArray = (Object[]) returnValue;
                for (int i = 0; i < returnArray.length; i++) {
                    if (i > 0) {
                        results.append(',');
                    }
                    results.append('[').append(i).append(']').append(returnArray[i]);
                }
            } else {
                results.append(returnValue);
            }
            return results.toString();
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public static String subscribe(String param1, String param2) {
        if (param1 == null) {
            return enableHelp();        // if no second arg, assume no context is provided
        }
        String context = param2 == null ? m_context : param1;

        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";        // get the filter for this context
        }
        javax.management.NotificationFilterSupport filter = (javax.management.NotificationFilterSupport) m_notificationFilters.get(context);
        if (filter == null) {
            filter = new javax.management.NotificationFilterSupport();        // add the notification types to the filter
        }
        String notifications = param2 == null ? param1 : param2;
        StringTokenizer st = new StringTokenizer(notifications, ",");
        while (st.hasMoreTokens()) {
            filter.enableType(st.nextToken());        // subscribe to the notifications
        }
        try {
            ObjectName name = new ObjectName(context);
            m_connector.addNotificationListener(name, m_notificationListener, filter, "");

            // save the notifications
            m_notificationFilters.put(context, filter);

            return null;
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }

    public static String unsubscribe(String param1, String param2) {
        if (param1 == null) {
            return enableHelp();        // if no second arg, assume no context is provided
        }
        String context = param2 == null ? m_context : param1;

        if (context.length() == 0 || !isComponentContext(context)) {
            return "Invalid working context for command.";        // get the filter for this context
        }
        javax.management.NotificationFilterSupport filter = (javax.management.NotificationFilterSupport) m_notificationFilters.get(context);
        if (filter == null) {
            filter = new javax.management.NotificationFilterSupport();        // add the notification types to the filter
        }
        String notifications = param2 == null ? param1 : param2;
        StringTokenizer st = new StringTokenizer(notifications, ",");
        while (st.hasMoreTokens()) {
            filter.disableType(st.nextToken());        // unsubscribe to the notifications
        }
        try {
            ObjectName name = new ObjectName(context);
            if (filter.getEnabledTypes().size() == 0) {
                m_connector.removeNotificationListener(name, m_notificationListener);
                m_notificationFilters.remove(context);
            } else // there are still notifications that should be subscribed to
            {
                m_connector.addNotificationListener(name, m_notificationListener, filter, "");
                m_notificationFilters.put(context, filter);
            }

            return null;
        } catch (Exception e) {
            return exceptionToString(e);
        }
    }
    //
    // command help strings
    //
    public static String exitHelp() {
        return "Usage: exit" +
                "\n\n" +
                "Disconnects from the domain and exits this application.";
    }

    public static String ccHelp() {
        return "Usage: cc <domain>[.<container>[:ID=<id>]]" +
                "\n\n" +
                "Change the current working context to that provided.";
    }

    public static String showAttributesHelp() {
        return "Usage: show attributes [<domain>.<container>:ID=<id>]" +
                "\n\n" +
                "Shows the management attribute types exposed by the given component context. If no component context is provided, the current context will be used.";
    }

    public static String showOperationsHelp() {
        return "Usage: show operations [<domain>.<container>:ID=<id>]" +
                "\n\n" +
                "Shows the management operations exposed by the given component context. If no component context is provided, the current context will be used.";
    }

    public static String showNotificationsHelp() {
        return "Usage: show notifications [<domain>.<container>:ID=<id>]" +
                "\n\n" +
                "Shows the management notification types exposed by the given component context. If no component context is provided, the current context will be used.";
    }

    public static String showMetricsHelp() {
        return "Usage: show metrics [<domain>.<container>:ID=<id>]" +
                "\n\n" +
                "Shows the metric types exposed by the given component context. If no component context is provided, the current context will be used.";
    }

    public static String showStatusHelp() {
        return "Usage: show status [<domain>{.<container>{:ID=<id>}}]" +
                "\n\n" +
                "Shows the runtime state of the domain, container or component specified by the given context. If no context is provided, the current context will be used.";
    }

    public static String showLogHelp() {
        return "Usage: show log [<domain>.<container>]" +
                "\n\n" +
                "Shows the tail (up to 200Kb) of the container log file. If no context is provided, the current context will be used.";
    }

    public static String showEnabledHelp() {
        return "Usage: show enabled [<domain>.<container>:ID=<id>]" +
                "\n\n" +
                "Shows the metrics currently enabled in the given component context. If no context is provided, the current context will be used.";
    }

    public static String getHelp() {
        return "Usage: get [<domain>.<container>:ID=<id>] <attributeName>{,<attributeName>,...}" +
                "\n\n" +
                "Gets the current values of the requested attributes for the given component context. If no component context is provided, the current context will be used.\n\nMultiple attribute names must be supplied as a comma seperated list enclosed in quotes.";
    }

    public static String setHelp() {
        return "Usage: set [<domain>.<container>:ID=<id>] <attributeName> <attribute value>" +
                "\n\n" +
                "Resets the value of the given attribute of the given component context. If no component context is provided, the current context will be used.";
    }

    public static String enableHelp() {
        return "Usage: enable [<domain>.<container>:ID=<id>] <metric name>{,<metric name>...}" +
                "\n\n" +
                "Enables the given metric(s) in the given component context. If no component context is provided, the current context will be used.\n\nMultiple metric names must be supplied as a comma seperated list.\n\nAssumes the given metric(s) is(are) dynamic (i.e. can be enabled/disabled).";
    }

    public static String disableHelp() {
        return "Usage: disable [<domain>.<container>:ID=<id>] <metric name>{,<metric name>...}" +
                "\n\n" +
                "Disables the given metric(s) in the given component context. If no component context is provided, the current context will be used.\n\nMultiple metric names must be supplied as a comma seperated list.\n\nAssumes the given metric(s) is(are) dynamic (i.e. can be enabled/disabled).";
    }

    public static String captureHelp() {
        return "Usage: capture [<domain>.<container>:ID=<id>] <metric name>{,<metric name>...}" +
                "\n\n" +
                "Captures and displays the values of given metric(s) in the given component context. If no component context is provided, the current context will be used.\n\nMultiple metric names must be supplied as a comma seperated list.";
    }

    public static String invokeHelp() {
        return "Usage: invoke [<domain>.<container>:ID=<id>] <operation name> {param1,{param2...}}" +
                "\n\n" +
                "Invokes an operation on the given component context. If no component context is provided, the current context will be used.\n\nMultiple parameters must be supplied as a comma seperated list (enclosed in quotes if spaces are required in individual parameter values).\n\nOnly operations that take base types as parameters are supported via this command. Operations with multiple signatures are not fully supported (the first operation name match that occurs will be used). Best attempts will be made to display operation return values (when available).";
    }

    public static String subscribeHelp() {
        return "Usage: subscribe [<domain>.<container>:ID=<id>] <notification type>{,<notification type>...}" +
                "\n\n" +
                "Subscribes to the given notification(s) on the given component context. If no component context is provided, the current context will be used.\n\nMultiple notification types must be supplied as a comma seperated list.";
    }

    public static String unsubscribeHelp() {
        return "Usage: unsubscribe [<domain>.<container>:ID=<id>] <notification type>{,<notification type>...}" +
                "\n\n" +
                "Subscribes to the given notification(s) on the given component context. If no component context is provided, the current context will be used.\n\nMultiple notification types must be supplied as a comma seperated list.";
    }
    //
    // other methods
    //
    private static String exceptionToString(Exception exception) {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        PrintWriter writer = new PrintWriter(out, true);
        printStackTrace(exception, writer);

        byte[] data = out.toByteArray();
        try {
            out.close();
        } catch (IOException io) {
            System.err.println("exA");
        }

        return new String(data);
    }

    private static void printStackTrace(Throwable exception, PrintWriter writer) {
        exception.printStackTrace(writer);
        Throwable target = null;
        if (exception instanceof InvocationTargetException) {
            target = ((InvocationTargetException) exception).getTargetException();
        }
        if (exception instanceof RuntimeOperationsException) {
            target = ((RuntimeOperationsException) exception).getTargetException();
        }
        if (exception instanceof RuntimeMBeanException) {
            target = ((RuntimeMBeanException) exception).getTargetException();
        }
        if (exception instanceof ReflectionException) {
            target = ((ReflectionException) exception).getTargetException();
        }
        if (exception instanceof MBeanException) {
            target = ((MBeanException) exception).getTargetException();
        }
        if (target != null) {
            writer.println("Caused by...");
            printStackTrace(target, writer);
        }
    }

    /**
     * Determine if the given string represents a component context by validating
     * the format is of the form x.y:ID=z (this is not an exact test).
     *
     * @param context
     * @return
     */
    private static boolean isComponentContext(String context) {
        int periodIndex = context.indexOf('.');
        if (periodIndex < 1) {
            return false;
        }
        int colonIndex = context.indexOf(":ID=");
        if (colonIndex < (periodIndex + 2)) {
            return false;
        }
        if (context.length() == (colonIndex + 4)) {
            return false;
        }
        return true;
    }

    private static String showComponentStatus(String componentContext)
            throws Exception {
        // establish the AGENT in the same container and get the state for the whole container
        ObjectName name = new ObjectName(componentContext);
        String container = name.getDomain();
        ObjectName agent = new ObjectName(container + ":ID=AGENT");
        IContainerState containerState = (IContainerState) m_connector.invoke(agent, "getContainerState", EMPTY_OBJECT_ARRAY, EMPTY_STRING_ARRAY);

        IComponentState[] componentStates = containerState.getComponentStates();
        IComponentState componentState = null;

        // find the component we have been requested
        String id = name.getKeyProperty("ID");
        for (int i = 0; i < componentStates.length; i++) {
            if (((IComponentIdentity) componentStates[i].getRuntimeIdentity()).getComponentName().equals(id)) {
                componentState = componentStates[i];
                break;
            }
        }

        // if not found report it
        if (componentState == null) {
            return "*unknown component*";
        }
        return formatContainerState(containerState, new IComponentState[]{componentState}).toString();
    }

    private static String showContainerStatus(String containerContext)
            throws Exception {
        // establish the AGENT in the same container and get the state for the whole container
        ObjectName agent = new ObjectName(containerContext + ":ID=AGENT");
        IContainerState containerState = (IContainerState) m_connector.invoke(agent, "getContainerState", EMPTY_OBJECT_ARRAY, EMPTY_STRING_ARRAY);

        return formatContainerState(containerState, containerState.getComponentStates()).toString();
    }

    private static String showDomainStatus(String domainContext)
            throws Exception {
        // establish the AGENT in the same container and get the state for the whole container
        ObjectName agentManager = new ObjectName(domainContext + ".AGENT MANAGER:ID=AGENT MANAGER");
        IState[] containerStates = (IState[]) m_connector.invoke(agentManager, "getCollectiveState", EMPTY_OBJECT_ARRAY, EMPTY_STRING_ARRAY);

        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < containerStates.length; i++) {
            if (i > 0) {
                buffer.append("\n");
            }
            IContainerState containerState = (IContainerState) containerStates[i];
            buffer.append(formatContainerState(containerState, containerState.getComponentStates()));
        }

        return buffer.toString();
    }

    private static StringBuffer formatContainerState(IContainerState containerState, IComponentState[] componentStates) {
        StringBuffer buffer = new StringBuffer(containerState.getRuntimeIdentity().getCanonicalName());
        buffer.append(" (state=").append(IContainerState.STATE_TEXT[containerState.getState()]);
        buffer.append(", host=").append(containerState.getContainerHost()).append(')');
        for (int i = 0; i < componentStates.length; i++) {
            buffer.append("\n- ");
            buffer.append(((IComponentIdentity) componentStates[i].getRuntimeIdentity()).getComponentName());
            buffer.append(" (state=").append(componentStates[i].getStateString()).append(')');
        }

        return buffer;
    }

    private static String formatUptimeString(long milli) {
        String uptime = "";
        int hours = (int) (milli / 3600000);
        if (hours < 10) {
            uptime = uptime + "0" + hours;
        } else {
            uptime = uptime + hours;
        }
        uptime = uptime + ":";
        int minutes = (int) (milli % 3600000) / 60000;
        if (minutes < 10) {
            uptime = uptime + "0" + minutes;
        } else {
            uptime = uptime + minutes;
        }
        uptime = uptime + ":";
        int seconds = ((int) (milli % 3600000) % 60000) / 1000;
        if (seconds < 10) {
            uptime = uptime + "0" + seconds;
        } else {
            uptime = uptime + seconds;
        }
        return uptime;
    }

    private static Object convertDatatype(Class newClass, String newValue)
            throws Exception {
        if (newClass.isArray()) {
            Class componentType = newClass.getComponentType();
            StringTokenizer st = new StringTokenizer(newValue, ",");
            int tokenCount = st.countTokens();
            Object newArray = Array.newInstance(componentType, tokenCount);
            for (int i = 0; i < tokenCount; i++) {
                Array.set(newArray, i, convertDatatype(componentType, st.nextToken()));
            }
            return newArray;
        } else // assume it has a constructor that takes a String
        {
            Constructor constructor = newClass.getConstructor(STRING_PARAMTYPES);
            return constructor.newInstance(new Object[]{newValue});
        }
    }

    private final static class NotificationReporter
            implements NotificationListener {

        @Override
        public void handleNotification(Notification notification, Object handback) {
            StringBuffer buffer = new StringBuffer();
            buffer.append('[').append(SIMPLE_DATE_FORMAT.format(new Date(notification.getTimeStamp()))).append(']');
            buffer.append(' ').append(notification.getType()).append(' ');
            buffer.append('(').append("source=").append(notification.getSource());

            // attributes
            Object[] attributes = ((INotification) notification).getAttributes().entrySet().toArray();
            for (int i = 0; i < attributes.length; i++) {
                buffer.append(", ");
                Map.Entry entry = (Map.Entry) attributes[i];
                buffer.append(entry.getKey()).append('=').append(entry.getValue());
            }

            buffer.append(')');

            OStream.println();
            OStream.println(buffer.toString());
            redoPrompt();
        }
    }
}
