package heuristicmonitor;

/**
 * Used for the demo (send a 1 million of JMS messages to the SampleQ1 queue)
 * @author  rares
 */

public class QSender {
    private static final String DEFAULT_BROKER_NAME = "tcp://localhost:2506";
    private static final String DEFAULT_PASSWORD = "Administrator";
    private static final String DEFAULT_USER = "Administrator";
    private static final int NUM_MESSAGES = 1000000;
    // milliseconds (30 minutes)
    private static final int    MESSAGE_LIFESPAN = 1800000;
    private static final String DEFAULT_QUEUE = "SampleQ1";


    private javax.jms.Connection connect = null;
    private javax.jms.Session sendSession = null;
    private javax.jms.MessageProducer sender = null;

    /** Create JMS client for sending and receiving messages. */
    private void talker( String broker, String username,
       String password, String sQueue) {
        // Create a connection.
        try
        {
            javax.jms.ConnectionFactory factory;
            factory = (new progress.message.jclient.ConnectionFactory (broker));
            connect = factory.createConnection (username, password);
            sendSession = connect.createSession(false,javax.jms.Session.AUTO_ACKNOWLEDGE);
        }
        catch (javax.jms.JMSException jmse)
        {
            System.err.println("error: Cannot connect to Broker - " + broker);
            jmse.printStackTrace();
            System.exit(1);
        }

        // Create Sender
        try
        {
            if (sQueue != null)
            {
                javax.jms.Queue sendQueue = sendSession.createQueue (sQueue);
                sender = sendSession.createProducer(sendQueue);
            }
            connect.start();
        }
        catch (javax.jms.JMSException jmse)
        {
            jmse.printStackTrace();
            exit();
        }
        try {
        javax.jms.TextMessage msg = sendSession.createTextMessage();
        msg.setText( 
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" + 
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" + 
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" +
            "ccccccccccccccccccccccccccccccccccccccccccccccccccccccc" 
            );
        // Queues usually are used for PERSISTENT messages.
        // Hold messages for 30 minutes (1,800,000 millisecs).

        for (int i=0;i<NUM_MESSAGES;i++) {
           sender.send( msg,
                      javax.jms.DeliveryMode.PERSISTENT,
                      javax.jms.Message.DEFAULT_PRIORITY,
                      MESSAGE_LIFESPAN);
        }
        }
        catch ( javax.jms.JMSException jmse )
        {
            jmse.printStackTrace();
        }
        // Close the connection.
        exit();
    }


    /** Cleanup resources and then exit. */
    private void exit()
    {
        try
        {
            connect.close();
        }
        catch (javax.jms.JMSException jmse)
        {
            jmse.printStackTrace();
        }
        System.out.println("Finish");
        System.exit(0);
    }

    /** Main program entry point. */
    public static void main(String argv[]) {
        QSender talk = new QSender();
        talk.talker (DEFAULT_BROKER_NAME, DEFAULT_USER,
                     DEFAULT_PASSWORD, DEFAULT_QUEUE);
    }


}