/*
 * Used to subscribe at the topic Sample.T1
 */

package heuristicmonitor;

/**
 *
 * @author rares
 */

public class TSubscriber
    implements javax.jms.MessageListener
{
    private static final String DEFAULT_BROKER_NAME = "localhost:2506";
    private static final String DEFAULT_PASSWORD = "Administrator";
    private static final String DEFAULT_USER = "Administrator";
    private static final String APP_TOPIC = "Sample.T02";

    private javax.jms.Connection connect = null;
    private javax.jms.Session subSession = null;

    /** Create JMS client for publishing and subscribing to messages. */
    private void chatter( String broker, String username, String password)
    {
        // Create a connection.
        try
        {
            javax.jms.ConnectionFactory factory;
            factory = (new progress.message.jclient.ConnectionFactory (broker));
            connect = factory.createConnection (username, password);
            subSession = connect.createSession(false,javax.jms.Session.AUTO_ACKNOWLEDGE);
        }
        catch (javax.jms.JMSException jmse)
        {
            System.err.println("error: Cannot connect to Broker - " + broker);
            jmse.printStackTrace();
            System.exit(1);
        }

        // Create Publisher and Subscriber to 'chat' topics
        try
        {
            javax.jms.Topic topic = subSession.createTopic (APP_TOPIC);
            javax.jms.MessageConsumer subscriber = subSession.createConsumer(topic);
            subscriber.setMessageListener(this);
            // Now that setup is complete, start the Connection
            connect.start();
        }
        catch (javax.jms.JMSException jmse)
        {
            jmse.printStackTrace();
        }

    }

    /**
     * Handle the message
     * (as specified in the javax.jms.MessageListener interface).
     */
    public void onMessage( javax.jms.Message aMessage)
    {
        try
        {
            // Cast the message as a text message.
            javax.jms.TextMessage textMessage = (javax.jms.TextMessage) aMessage;

            // This handler reads a single String from the
            // message and prints it to the standard output.
            try
            {
                String string = textMessage.getText();
                System.out.println( string );
            }
            catch (javax.jms.JMSException jmse)
            {
                jmse.printStackTrace();
            }
        }
        catch (java.lang.RuntimeException rte)
        {
            rte.printStackTrace();
        }
    }

    /** Cleanup resources and then exit. */
    private void exit()
    {
        try
        {
            connect.close();
        }
        catch (javax.jms.JMSException jmse)
        {
            jmse.printStackTrace();
        }

        System.exit(0);
    }

    //
    // NOTE: the remainder of this sample deals with reading arguments
    // and does not utilize any JMS classes or code.
    //

    /** Main program entry point. */
    public static void main(String argv[]) {


        // Values to be read from parameters
        String broker    = DEFAULT_BROKER_NAME;
        String username  = DEFAULT_USER;
        String password  = DEFAULT_PASSWORD;

        // Start the JMS client for the "chat".
        TSubscriber chat = new TSubscriber();
        chat.chatter (broker, username, password);

    }

    
}//class
