package heuristicmonitor;
import processing.core.*;

public class cubic extends PApplet {

    /**
     * Cubic Grid. 
     * By Rares Stanciulescu
     *
     * 3D translucent colored grid
     * for SonicMQ Heuristic Monitor
     */
    float boxSize = 40;
    float margin = boxSize * 2;
    float depth = 400;
    int boxFill;
    String[] lines;
    int index = 0;
    static String fName = null;
    
    public void setup() {
        size(200, 200, P3D);
        noStroke();
    }

    public void draw() {
        background(255);
        // center and spin grid
        translate(width / 2, height / 2, -depth / 2);
        rotateY(frameCount * PI / 300);
        rotateX(frameCount * PI / 300);
        Integer a;
        Integer b;
        Integer c;
        Integer d;
        int numBoxes = 1;
        int dimensions = 0;
        String[] pieces;
        try {
         lines = loadStrings(fName);
        } catch (Exception ex){
          return;   
        } 
         
        if (lines==null) {
           return;
        }
        
        String[] lastLine = null;
        try {
            //get the number of dimensions
            dimensions = (int) new Integer(lines[0]);
            lastLine = split(lines[lines.length - 1], ' ');
        } catch (Exception ex) {
            return;
        }    
        Integer aL = new Integer(lastLine[1]);
        Integer bL = new Integer(lastLine[2]);
        Integer cL = new Integer(lastLine[3]);
        
        int sum  = 0;
        for (int i = 1;i<dimensions;i++) {
            sum = sum + new Integer(lastLine[i]);
        }
        Integer sumL = new Integer(sum);

        // build grid using multiple translations 
        for (float i = -depth / 2 + margin; i <= depth / 2 - margin; i += boxSize) {
            pushMatrix();

            for (float j = -height + margin; j <= height - margin; j += boxSize) {
                pushMatrix();
                for (float k = -width + margin; k <= width - margin; k += boxSize) {
                    // base fill color on counter values, abs function 
                    // ensures values stay within legal range
                    //boxFill = color(abs(i), abs(j), abs(k), 50);

                    try {
                        pieces = split(lines[lines.length - numBoxes], ' ');
                        a = new Integer(pieces[1]);
                        if(aL > 0 )
                         a = abs(((255 * a) / aL));
                        b = new Integer(pieces[2]);
                        if(bL > 0 )
                          b = abs(((255 * b) / bL));
                        c = new Integer(pieces[3]);
                        if(cL > 0 )
                        c = abs(((255 * c) / cL));

                        sum  = 0;
                        for (int w = 1;w<dimensions;w++) {
                            sum = sum + new Integer(pieces[w]);
                        }
                        d = new Integer(sum);
                        if(sumL > 0 )
                        d = abs(((255 * sum) / sumL));
                        boxFill = color(
                                abs(a.intValue() - d.intValue()),
                                abs(b.intValue() - d.intValue()),
                                abs(c.intValue() - d.intValue()), 50);
                        
                    } catch (Exception ex) {
                        boxFill = color(0, 0, 0, 50);
                    }
                    numBoxes++;
                    if (numBoxes >= boxSize) {
                        numBoxes = 1;
                    }

                    pushMatrix();
                    translate(k, j, i);
                    fill(boxFill);
                    box(boxSize, boxSize, boxSize);
                    popMatrix();
                }
                popMatrix();
            }
            popMatrix();
        }
    }
    
    

    static public void main(String args[]) {
         fName = args[1];
         PApplet.main(new String[] { args[0] });
    }
}